@@ -21,6 +21,6 @@ To start any OpenLLM model:
     openllm start <model_name> --options ...
 """
 if __name__ == "__main__":
-    from openllm.cli import create_cli
+    from openllm.cli import cli
 
-    create_cli()()
+    cli()
