@@ -15,22 +15,25 @@
 from __future__ import annotations
 
 import copy
-import enum
 import functools
 import logging
 import os
 import re
+import sys
 import types
 import typing as t
 from abc import ABC, ABCMeta, abstractmethod
 
 import bentoml
 import inflection
-from bentoml.types import ModelSignature, ModelSignatureDict
+import orjson
+from bentoml._internal.models.model import ModelSignature
+from bentoml._internal.types import ModelSignatureDict
 
 import openllm
 
 from .exceptions import ForbiddenAttributeError, OpenLLMException
+from .models.auto import AutoConfig
 from .utils import ENV_VARS_TRUE_VALUES, LazyLoader, bentoml_cattr
 
 if t.TYPE_CHECKING:
@@ -38,10 +41,9 @@ if t.TYPE_CHECKING:
     import transformers
     from bentoml._internal.runner.strategy import Strategy
 
-    from ._types import LLMModel, LLMTokenizer
-
     class LLMRunner(bentoml.Runner):
-        llm: openllm.LLM
+        llm: openllm.LLM[t.Any, t.Any]
+        config: openllm.LLMConfig
         llm_type: str
         identifying_params: dict[str, t.Any]
 
@@ -50,11 +52,10 @@ if t.TYPE_CHECKING:
 
 else:
     transformers = LazyLoader("transformers", globals(), "transformers")
+    torch = LazyLoader("torch", globals(), "torch")
 
 logger = logging.getLogger(__name__)
 
-_object_setattr = object.__setattr__
-
 # NOTE: `1-2` -> text-generation and text2text-generation
 FRAMEWORK_TO_AUTOCLASS_MAPPING = {
     "pt": ("AutoModelForCausalLM", "AutoModelForSeq2SeqLM"),
@@ -71,35 +72,15 @@ def convert_transformers_model_name(name: str) -> str:
     return re.sub("[^a-zA-Z0-9]+", "-", name)
 
 
-class TypeMeta(enum.EnumMeta):
-    def __getitem__(self, key: str) -> enum.Enum:
-        # Type safe getters
-        normalised = key.replace("-", "_").upper()
-        try:
-            return self._member_map_[normalised]
-        except KeyError:
-            raise OpenLLMException(
-                f"TaskType '{key}' is not yet supported. Current supported tasks: {set(self._member_map_)}"
-            ) from None
-        except TypeError:
-            raise OpenLLMException(f"getitem key must be a string. Got {type(key)} instead.") from None
-
-
-@enum.unique
-class TaskType(enum.Enum, metaclass=TypeMeta):
-    TEXT_GENERATION = enum.auto()
-    TEXT2TEXT_GENERATION = enum.auto()
-
-
 def import_model(
-    model_name: str,
+    model_id: str,
     tag: bentoml.Tag,
     _model_framework: str,
     *model_args: t.Any,
     tokenizer_kwds: dict[str, t.Any],
     **attrs: t.Any,
 ):
-    """Auto detect model type from given model_name and import it to bentoml's model store.
+    """Auto detect model type from given model_id and import it to bentoml's model store.
 
     For all kwargs, it will be parsed into `transformers.AutoConfig.from_pretrained` first,
     returning all of the unused kwargs.
@@ -111,7 +92,7 @@ def import_model(
     Refer to Transformers documentation for more information about kwargs.
 
     Args:
-        model_name: Model name to be imported. use `openllm models` to see available entries
+        model_id: Model id to be imported. See `openllm models` for all supported models.
         tag: Tag to be used for the model. This is usually generated for you.
         model_args: Args to be passed into AutoModelForSeq2SeqLM or AutoModelForCausalLM (+ TF, Flax variants).
         **attrs: Kwargs to be passed into AutoModelForSeq2SeqLM or AutoModelForCausalLM (+ TF, Flax variants).
@@ -139,71 +120,66 @@ def import_model(
         config, attrs = t.cast(
             "tuple[transformers.PretrainedConfig, dict[str, t.Any]]",
             transformers.AutoConfig.from_pretrained(
-                model_name, return_unused_kwargs=True, trust_remote_code=trust_remote_code, **hub_attrs, **copied_attrs
+                model_id, return_unused_kwargs=True, trust_remote_code=trust_remote_code, **hub_attrs, **copied_attrs
             ),
         )
 
     if type(config) in transformers.MODEL_FOR_CAUSAL_LM_MAPPING:
-        task_type = "text-generation"
+        idx = 0
     elif type(config) in transformers.MODEL_FOR_SEQ_TO_SEQ_CAUSAL_LM_MAPPING:
-        task_type = "text2text-generation"
+        idx = 1
     else:
         raise OpenLLMException(f"Model type {type(config)} is not supported yet.")
 
-    return bentoml.transformers.save_model(
-        tag,
-        getattr(
-            transformers, FRAMEWORK_TO_AUTOCLASS_MAPPING[_model_framework][TaskType[task_type].value - 1]
-        ).from_pretrained(
-            model_name, *model_args, config=config, trust_remote_code=trust_remote_code, **hub_attrs, **attrs
-        ),
-        custom_objects={
-            "tokenizer": t.cast(
-                "LLMTokenizer",
-                transformers.AutoTokenizer.from_pretrained(
-                    model_name, config=config, trust_remote_code=trust_remote_code, **hub_attrs, **tokenizer_kwds
-                ),
-            )
-        },
-    )
+    try:
+        return bentoml.transformers.save_model(
+            tag,
+            getattr(
+                transformers,
+                FRAMEWORK_TO_AUTOCLASS_MAPPING[_model_framework][idx],
+            ).from_pretrained(
+                model_id,
+                *model_args,
+                config=config,
+                trust_remote_code=trust_remote_code,
+                **hub_attrs,
+                **attrs,
+            ),
+            custom_objects={
+                "tokenizer": transformers.AutoTokenizer.from_pretrained(
+                    model_id,
+                    config=config,
+                    trust_remote_code=trust_remote_code,
+                    **hub_attrs,
+                    **tokenizer_kwds,
+                )
+            },
+        )
+    finally:
+        import gc
+
+        gc.collect()
+
+        # NOTE: We need to free up the cache after importing the model
+        # in the case where users first run openllm start without the model
+        # available locally.
+        if openllm.utils.is_torch_available() and torch.cuda.is_available():
+            torch.cuda.empty_cache()
 
 
-_required_namespace = {"default_model", "variants"}
-
-_reserved_namespace = _required_namespace | {
-    "config_class",
-    "model",
-    "tokenizer",
-    "import_kwargs",
-    "requirements",
-}
+_reserved_namespace = {"config_class", "model", "tokenizer", "import_kwargs"}
 
 
 class LLMInterface(ABC):
     """This defines the loose contract for all openllm.LLM implementations."""
 
-    default_model: str
-    """Return the default model to use when using 'openllm start <model_name>'.
-    This could be one of the keys in 'self.variants' or custom users model."""
-
-    variants: list[str]
-    """A list of supported pretrained models tag for this given runnable.
-
-    For example:
-        For FLAN-T5 impl, this would be ["google/flan-t5-small", "google/flan-t5-base",
-                                            "google/flan-t5-large", "google/flan-t5-xl", "google/flan-t5-xxl"]
-    """
-
     config_class: type[openllm.LLMConfig]
     """The config class to use for this LLM. If you are creating a custom LLM, you must specify this class."""
 
-    import_kwargs: dict[str, t.Any] | None = None
-    """The default import kwargs to used when importing the model. 
-    This will be passed into 'openllm.LLM.import_model'."""
-
-    requirements: list[str] | None = None
-    """The default PyPI requirements needed to run this given LLM. By default, we will depend on 
-    bentoml, torch, transformers."""
+    @property
+    def import_kwargs(self) -> dict[str, t.Any] | None:
+        """The default import kwargs to used when importing the model.
+        This will be passed into 'openllm.LLM.import_model'."""
 
     @abstractmethod
     def generate(self, prompt: str, **preprocess_generate_kwds: t.Any) -> t.Any:
@@ -224,11 +200,7 @@ class LLMInterface(ABC):
 
         It takes a prompt that is given by the user, attrs that can be parsed with the prompt.
 
-        NOTE: the attrs should also handle the following default attributes from all LLMConfig:
-        - use_default_prompt_template
-
         Returns a tuple of three items:
-        - The processed prompt text depending on `use_default_prompt_template`
         - The attributes dictionary that can be passed into LLMConfig to generate a GenerationConfig
         - The attributes dictionary that will be passed into `self.postprocess_generate`.
         """
@@ -251,11 +223,35 @@ class LLMInterface(ABC):
         pass
 
     def import_model(
-        self, pretrained: str, tag: bentoml.Tag, *args: t.Any, tokenizer_kwds: dict[str, t.Any], **attrs: t.Any
+        self, model_id: str, tag: bentoml.Tag, *args: t.Any, tokenizer_kwds: dict[str, t.Any], **attrs: t.Any
     ) -> bentoml.Model:
         """This function can be implemented if default import_model doesn't satisfy your needs."""
         raise NotImplementedError
 
+    def load_model(self, tag: bentoml.Tag, *args: t.Any, **attrs: t.Any) -> t.Any:
+        """This function can be implemented to override th default load_model behaviour. See falcon for
+        example implementation."""
+        raise NotImplementedError
+
+
+def _default_post_init(self: LLM[t.Any, t.Any]):
+    # load_in_mha: Whether to apply BetterTransformer (or Torch MultiHeadAttention) during inference load.
+    #              See https://pytorch.org/blog/a-better-transformer-for-fast-transformer-encoder-inference/
+    #              for more information.
+    # NOTE: set a default variable to transform to BetterTransformer by default for inference
+    self.load_in_mha = (
+        os.environ.get(self.config_class.__openllm_env__.bettertransformer, str(False)).upper() in ENV_VARS_TRUE_VALUES
+    )
+    if self.config_class.__openllm_requires_gpu__:
+        # For all models that requires GPU, no need to offload it to BetterTransformer
+        # use bitsandbytes instead
+
+        self.load_in_mha = False
+
+
+_M = t.TypeVar("_M")
+_T = t.TypeVar("_T")
+
 
 class LLMMetaclass(ABCMeta):
     def __new__(
@@ -263,60 +259,71 @@ class LLMMetaclass(ABCMeta):
     ) -> type:
         """Metaclass for creating a LLM."""
         if LLMInterface not in bases:  # only actual openllm.LLM should hit this branch.
-            if "__annotations__" not in namespace:
-                annotations_dict: dict[str, t.Any] = {}
-                namespace["__annotations__"] = annotations_dict
-
-            # NOTE: check for required attributes
-            for k in _required_namespace:
-                if k not in namespace:
-                    raise RuntimeError(f"Missing required key '{k}'. Make sure to define it within the LLM subclass.")
+            annotations_dict: dict[str, t.Any] = {}
 
             # NOTE: set implementation branch
             prefix_class_name_config = cls_name
-            if "__llm_implementation__" in namespace:
-                raise RuntimeError(
-                    f"""\
-                __llm_implementation__ should not be set directly. Instead make sure that your class
-                name follows the convention prefix: 
-                - For Tensorflow implementation: 'TF{cls_name}'
-                - For Flax implementation: 'Flax{cls_name}'
-                - For PyTorch implementation: '{cls_name}'"""
-                )
             if cls_name.startswith("Flax"):
                 implementation = "flax"
                 prefix_class_name_config = cls_name[4:]
+                annotations_dict.update(
+                    {
+                        "__llm_model__": "transformers.FlaxPreTrainedModel",
+                        "model": "transformers.FlaxPreTrainedModel",
+                    }
+                )
             elif cls_name.startswith("TF"):
                 implementation = "tf"
                 prefix_class_name_config = cls_name[2:]
+                annotations_dict.update(
+                    {
+                        "__llm_model__": "transformers.TFPreTrainedModel",
+                        "model": "transformers.TFPreTrainedModel",
+                    }
+                )
             else:
                 implementation = "pt"
+                annotations_dict.update(
+                    {
+                        "__llm_model__": "transformers.PreTrainedModel",
+                        "model": "transformers.PreTrainedModel",
+                    }
+                )
             namespace["__llm_implementation__"] = implementation
 
+            config_class = AutoConfig.infer_class_from_name(prefix_class_name_config)
+            annotations_dict["config_class"] = f"type[openllm.{config_class.__qualname__}]"
+            annotations_dict["config"] = f"openllm.{config_class.__qualname__}"
+
             # NOTE: setup config class branch
             if "__openllm_internal__" in namespace:
                 # NOTE: we will automatically find the subclass for this given config class
                 if "config_class" not in namespace:
                     # this branch we will automatically get the class
-                    namespace["config_class"] = getattr(openllm, f"{prefix_class_name_config}Config")
+                    namespace["config_class"] = config_class
                 else:
                     logger.debug(f"Using config class {namespace['config_class']} for {cls_name}.")
+            # NOTE: check for required attributes
+            else:
+                if "config_class" not in namespace:
+                    raise RuntimeError(
+                        "Missing required key 'config_class'. Make sure to define it within the LLM subclass."
+                    )
 
-            config_class: type[openllm.LLMConfig] = namespace["config_class"]
+            # NOTE: the llm_post_init branch
+            if "llm_post_init" in namespace:
+                original_llm_post_init = namespace["llm_post_init"]
 
-            # NOTE: update the annotations for self.config
-            namespace["__annotations__"]["config"] = t.get_type_hints(config_class)
+                def wrapped_llm_post_init(self: LLM[t.Any, t.Any]) -> None:
+                    """We need to both initialize private attributes and call the user-defined model_post_init
+                    method.
+                    """
+                    _default_post_init(self)
+                    original_llm_post_init(self)
 
-            for key in ("__openllm_start_name__", "__openllm_requires_gpu__"):
-                namespace[key] = getattr(config_class, key)
-
-            # NOTE: set a default variable to transform to BetterTransformer by default for inference
-            namespace["load_in_mha"] = namespace.get("load_in_mha", implementation in ("pt",))
-
-            if namespace["__openllm_requires_gpu__"]:
-                # For all models that requires GPU, no need to offload it to BetterTransformer
-                # use bitsandbytes instead
-                namespace["load_in_mha"] = False
+                namespace["llm_post_init"] = wrapped_llm_post_init
+            else:
+                namespace["llm_post_init"] = _default_post_init
 
             # NOTE: import_model branch
             if "import_model" not in namespace:
@@ -325,47 +332,60 @@ class LLMMetaclass(ABCMeta):
             else:
                 logger.debug("Using custom 'import_model' for %s", cls_name)
 
-            # NOTE: populate with default cache.
-            namespace.update({k: None for k in ("__llm_bentomodel__", "__llm_model__", "__llm_tokenizer__")})
+            namespace.update({k: None for k in {"__llm_model__", "__llm_tokenizer__", "__llm_bentomodel__"}})
+            tokenizer_type = "typing.Optional[transformers.PreTrainedTokenizerFast]"
+            annotations_dict.update(
+                {
+                    "load_in_mha": "bool",
+                    "tokenizer": tokenizer_type,
+                    "__llm_tokenizer__": tokenizer_type,
+                    "__llm_bentomodel__": "typing.Optional[bentoml.Model]",
+                }
+            )
+            # NOTE: transformers to ForwardRef
+            for key, value in annotations_dict.items():
+                if (3, 10) > sys.version_info >= (3, 9, 8) or sys.version_info >= (3, 10, 1):
+                    annotations_dict[key] = t.ForwardRef(value, is_argument=False, is_class=True)
+                else:
+                    annotations_dict[key] = t.ForwardRef(value, is_argument=False)
+
+            namespace["__annotations__"] = annotations_dict
+
+            cls: type[LLM[t.Any, t.Any]] = super().__new__(
+                t.cast("type[type[LLM[t.Any, t.Any]]]", mcls), cls_name, bases, namespace, **attrs
+            )
 
-            cls: type[LLM] = super().__new__(t.cast("type[type[LLM]]", mcls), cls_name, bases, namespace, **attrs)
             cls.__openllm_post_init__ = None if cls.llm_post_init is LLMInterface.llm_post_init else cls.llm_post_init
-
-            if getattr(cls, "config_class") is None:
-                raise RuntimeError(f"'config_class' must be defined for '{cls.__name__}'")
+            cls.__openllm_custom_load__ = None if cls.load_model is LLMInterface.load_model else cls.load_model
             return cls
         else:
             # the LLM class itself being created, no need to setup
             return super().__new__(mcls, cls_name, bases, namespace, **attrs)
 
 
-class LLM(LLMInterface, metaclass=LLMMetaclass):
+class LLM(LLMInterface, t.Generic[_M, _T], metaclass=LLMMetaclass):
     if t.TYPE_CHECKING:
         # NOTE: the following will be populated by metaclass
-        __llm_bentomodel__: bentoml.Model | None = None
-        __llm_model__: LLMModel | None = None
-        __llm_tokenizer__: LLMTokenizer | None = None
         __llm_implementation__: t.Literal["pt", "tf", "flax"]
-        __llm_kwargs__: dict[str, t.Any]
-        __llm_args__: tuple[t.Any, ...]
 
-        __openllm_start_name__: str
-        __openllm_requires_gpu__: bool
         __openllm_post_init__: t.Callable[[t.Self], None] | None
+        __openllm_custom_load__: t.Callable[[t.Self, t.Any, t.Any], None] | None
+
+        load_in_mha: bool
+        _llm_attrs: dict[str, t.Any]
+        _llm_args: tuple[t.Any, ...]
 
     # NOTE: the following is the similar interface to HuggingFace pretrained protocol.
-
     @classmethod
     def from_pretrained(
-        cls, pretrained: str | None = None, llm_config: openllm.LLMConfig | None = None, *args: t.Any, **attrs: t.Any
-    ) -> LLM:
-        return cls(pretrained=pretrained, llm_config=llm_config, *args, **attrs)
+        cls, model_id: str | None = None, llm_config: openllm.LLMConfig | None = None, *args: t.Any, **attrs: t.Any
+    ) -> LLM[_M, _T]:
+        return cls(model_id=model_id, llm_config=llm_config, *args, **attrs)
 
     def __init__(
         self,
-        pretrained: str | None = None,
+        model_id: str | None = None,
         llm_config: openllm.LLMConfig | None = None,
-        load_in_mha: bool | None = None,
         *args: t.Any,
         **attrs: t.Any,
     ):
@@ -388,7 +408,7 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
         ```python
         def import_model(
             self,
-            pretrained: str,
+            model_id: str,
             tag: bentoml.Tag,
             *args: t.Any,
             tokenizer_kwds: dict[str, t.Any],
@@ -397,11 +417,11 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
             return bentoml.transformers.save_model(
                 tag,
                 transformers.AutoModelForCausalLM.from_pretrained(
-                    pretrained, device_map="auto", torch_dtype=torch.bfloat16, **attrs
+                    model_id, device_map="auto", torch_dtype=torch.bfloat16, **attrs
                 ),
                 custom_objects={
                     "tokenizer": transformers.AutoTokenizer.from_pretrained(
-                        pretrained, padding_size="left", **tokenizer_kwds
+                        model_id, padding_size="left", **tokenizer_kwds
                     )
                 },
             )
@@ -420,23 +440,40 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
         Note: If you implement your own `import_model`, then `import_kwargs` will be the
         default kwargs for every load. You can still override those via ``openllm.Runner``.
 
-        Note that this tag will be generated based on `self.default_model` or the given `pretrained` kwds.
+        Note that this tag will be generated based on `self.default_id` or the given `pretrained` kwds.
         passed from the __init__ constructor.
 
-        ``llm_post_init`` can also be implemented if you need to do any
-        additional initialization after everything is setup.
+        ``llm_post_init`` can also be implemented if you need to do any additional
+        initialization after everything is setup.
+
+        Note: If you need to implement a custom `load_model`, the following is an example from Falcon implementation:
+
+        ```python
+        def load_model(self, tag: bentoml.Tag, *args: t.Any, **attrs: t.Any) -> t.Any:
+            torch_dtype = attrs.pop("torch_dtype", torch.bfloat16)
+            device_map = attrs.pop("device_map", "auto")
+
+            _ref = bentoml.transformers.get(tag)
+
+            model = bentoml.transformers.load_model(_ref, device_map=device_map, torch_dtype=torch_dtype, **attrs)
+            return transformers.pipeline("text-generation", model=model, tokenizer=_ref.custom_objects["tokenizer"])
+        ```
 
         Args:
-            pretrained: The pretrained model to use. Defaults to None. It will use 'self.default_model' if None.
-            llm_config: The config to use for this LLM. Defaults to None. If not passed, we will use 'self.config_class'
-                        to construct default configuration.
-            load_in_mha: Whether to apply BetterTransformer (or Torch MultiHeadAttention) during inference load.
-                         See https://pytorch.org/blog/a-better-transformer-for-fast-transformer-encoder-inference/
-                         for more information.
+            model_id: The pretrained model to use. Defaults to None. If None, 'self.default_id' will be used.
+            llm_config: The config to use for this LLM. Defaults to None. If not passed, OpenLLM
+                        will use `config_class` to construct default configuration.
             *args: The args to be passed to the model.
             **attrs: The kwargs to be passed to the model.
+
+        The following are optional:
+            openllm_model_version: version for this `model_id`. By default, users can ignore this if using pretrained
+                                   weights as OpenLLM will use the commit_hash of given model_id.
+                                   However, if `model_id` is a path, this argument is recomended to include.
         """
 
+        load_in_mha = attrs.pop("load_in_mha", False)
+
         if llm_config is not None:
             logger.debug("Using given 'llm_config=(%s)' to initialize LLM", llm_config)
             self.config = llm_config
@@ -445,27 +482,23 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
             # The rests of the kwargs that is not used by the config class should be stored into __openllm_extras__.
             attrs = self.config.__openllm_extras__
 
-        if pretrained is None:
-            pretrained = os.environ.get(self.config.__openllm_env__.pretrained, None)
-            if not pretrained:
-                assert self.default_model, "A default model is required for any LLM."
-                pretrained = self.default_model
+        if model_id is None:
+            model_id = os.environ.get(self.config.__openllm_env__.model_id, self.config.__openllm_default_id__)
+            assert model_id is not None
 
         # NOTE: This is the actual given path or pretrained weight for this LLM.
-        self._pretrained = pretrained
+        self._model_id = model_id
 
         # NOTE: Save the args and kwargs for latter load
-        self.__llm_args__ = args
-        self.__llm_kwargs__ = attrs
-
-        if load_in_mha is not None:
-            self.load_in_mha = (
-                os.environ.get(self.config.__openllm_env__.bettertransformer, str(load_in_mha)).upper()
-                in ENV_VARS_TRUE_VALUES
-            )
+        self._llm_args = args
+        self._llm_attrs = attrs
 
         if self.__openllm_post_init__:
-            self.__openllm_post_init__(self)
+            self.llm_post_init()
+
+        # finally: we allow users to overwrite the load_in_mha defined by the LLM subclass.
+        if load_in_mha:
+            self.load_in_mha = load_in_mha
 
     def __setattr__(self, attr: str, value: t.Any):
         if attr in _reserved_namespace:
@@ -477,25 +510,26 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
 
         super().__setattr__(attr, value)
 
+    @property
+    def model_id(self) -> str:
+        return self._model_id
+
     # NOTE: The section below defines a loose contract with langchain's LLM interface.
     @property
     def llm_type(self) -> str:
-        return convert_transformers_model_name(self._pretrained)
+        return convert_transformers_model_name(self._model_id)
 
     @property
     def identifying_params(self) -> dict[str, t.Any]:
-        return {"configuration": self.config.model_dump(), "variants": self.variants}
-
-    @property
-    def _bentomodel(self) -> bentoml.Model:
-        if self.__llm_bentomodel__ is None:
-            self.__llm_bentomodel__ = self.ensure_pretrained_exists()
-        return self.__llm_bentomodel__
+        return {
+            "configuration": self.config.model_dump_json().decode(),
+            "model_ids": orjson.dumps(self.config.__openllm_model_ids__).decode(),
+        }
 
     @t.overload
     def make_tag(
         self,
-        model_name_or_path: str | None = None,
+        model_id: str | None = None,
         return_unused_kwargs: t.Literal[False] = ...,
         trust_remote_code: bool = ...,
         **attrs: t.Any,
@@ -505,7 +539,7 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
     @t.overload
     def make_tag(
         self,
-        model_name_or_path: str | None = None,
+        model_id: str | None = None,
         return_unused_kwargs: t.Literal[True] = ...,
         trust_remote_code: bool = ...,
         **attrs: t.Any,
@@ -514,7 +548,7 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
 
     def make_tag(
         self,
-        model_name_or_path: str | None = None,
+        model_id: str | None = None,
         return_unused_kwargs: bool = False,
         trust_remote_code: bool = False,
         **attrs: t.Any,
@@ -535,8 +569,8 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
         Returns:
             A tuple of ``bentoml.Tag`` and a dict of unused kwargs.
         """
-        if model_name_or_path is None:
-            model_name_or_path = self._pretrained
+        if model_id is None:
+            model_id = self._model_id
 
         if "return_unused_kwargs" in attrs:
             logger.debug("Ignoring 'return_unused_kwargs' in 'generate_tag_from_model_name'.")
@@ -545,19 +579,19 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
         config, attrs = t.cast(
             "tuple[transformers.PretrainedConfig, dict[str, t.Any]]",
             transformers.AutoConfig.from_pretrained(
-                model_name_or_path, return_unused_kwargs=True, trust_remote_code=trust_remote_code, **attrs
+                model_id, return_unused_kwargs=True, trust_remote_code=trust_remote_code, **attrs
             ),
         )
-        name = convert_transformers_model_name(model_name_or_path)
+        name = convert_transformers_model_name(model_id)
 
-        if os.path.exists(os.path.dirname(model_name_or_path)):
+        if os.path.exists(os.path.dirname(model_id)):
             # If the model_name_or_path is a path, we assume it's a local path,
             # then users must pass a version for this.
             model_version = attrs.pop("openllm_model_version", None)
             if model_version is None:
                 logger.warning(
                     """\
-            When passing a path, it is recommended to also pass 'openllm_model_version' 
+            When passing a path, it is recommended to also pass 'openllm_model_version'
             into Runner/AutoLLM intialization.
 
             For example:
@@ -570,7 +604,7 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
             >>> import openllm
             >>> model = openllm.AutoLLM.for_model('/path/to/fine-tuning/model', openllm_model_version='lora-version')
 
-            No worries, OpenLLM will generate one for you. But for your own convenience, make sure to 
+            No worries, OpenLLM will generate one for you. But for your own convenience, make sure to
             specify 'openllm_model_version'.
             """
                 )
@@ -582,7 +616,7 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
                     "Given %s from '%s' doesn't contain a commit hash. We will generate"
                     " the tag without specific version.",
                     t.cast("type[transformers.PretrainedConfig]", config.__class__),
-                    model_name_or_path,
+                    model_id,
                 )
         tag = bentoml.Tag.from_taglike(f"{self.__llm_implementation__}-{name}:{model_version}")
 
@@ -590,9 +624,9 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
             return tag, attrs
         return tag
 
-    def ensure_pretrained_exists(self):
-        trust_remote_code = self.__llm_kwargs__.pop("trust_remote_code", self.config.__openllm_trust_remote_code__)
-        tag, kwds = self.make_tag(return_unused_kwargs=True, trust_remote_code=trust_remote_code, **self.__llm_kwargs__)
+    def ensure_model_id_exists(self) -> bentoml.Model:
+        trust_remote_code = self._llm_attrs.pop("trust_remote_code", self.config.__openllm_trust_remote_code__)
+        tag, kwds = self.make_tag(return_unused_kwargs=True, trust_remote_code=trust_remote_code, **self._llm_attrs)
         try:
             return bentoml.transformers.get(tag)
         except bentoml.exceptions.BentoMLException:
@@ -611,27 +645,37 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
                     **{k: v for k, v in self.import_kwargs.items() if not k.startswith("_tokenizer_")},
                     **kwds,
                 }
+
             return self.import_model(
-                self._pretrained,
+                self._model_id,
                 tag,
-                *self.__llm_args__,
+                *self._llm_args,
                 tokenizer_kwds=tokenizer_kwds,
                 trust_remote_code=trust_remote_code,
                 **kwds,
             )
+        finally:
+            if openllm.utils.is_torch_available() and torch.cuda.is_available():
+                torch.cuda.empty_cache()
+
+    @property
+    def _bentomodel(self) -> bentoml.Model:
+        if self.__llm_bentomodel__ is None:
+            self.__llm_bentomodel__ = self.ensure_model_id_exists()
+        return self.__llm_bentomodel__
 
     @property
     def tag(self) -> bentoml.Tag:
         return self._bentomodel.tag
 
     @property
-    def model(self) -> LLMModel | torch.nn.Module:
+    def model(self) -> _M:
         """The model to use for this LLM. This shouldn't be set at runtime, rather let OpenLLM handle it."""
         # Run check for GPU
-        trust_remote_code = self.__llm_kwargs__.pop("trust_remote_code", self.config.__openllm_trust_remote_code__)
-        self.config.check_if_gpu_is_available(self.__llm_implementation__)
+        trust_remote_code = self._llm_attrs.pop("trust_remote_code", self.config.__openllm_trust_remote_code__)
+        self.config.check_if_gpu_is_available(implementation=self.__llm_implementation__)
 
-        kwds = {k: v for k, v in self.__llm_kwargs__.items() if not k.startswith("_tokenizer_")}
+        kwds = {k: v for k, v in self._llm_attrs.items() if not k.startswith("_tokenizer_")}
 
         if self.import_kwargs:
             kwds = {**{k: v for k, v in self.import_kwargs.items() if not k.startswith("_tokenizer_")}, **kwds}
@@ -642,8 +686,10 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
             kwds["accelerator"] = "bettertransformer"
 
         if self.__llm_model__ is None:
-            # Hmm, bentoml.transformers.load_model doesn't yet support args.
-            self.__llm_model__ = self._bentomodel.load_model(*self.__llm_args__, **kwds)
+            if self.__openllm_custom_load__:
+                self.__llm_model__ = self.load_model(self.tag, *self._llm_args, **kwds)
+            else:
+                self.__llm_model__ = self._bentomodel.load_model(*self._llm_args, **kwds)
 
         if (
             self.load_in_mha
@@ -654,13 +700,11 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
             from optimum.bettertransformer import BetterTransformer
 
             self.__llm_model__ = BetterTransformer.transform(self.__llm_model__)
-        return self.__llm_model__
+        return t.cast(_M, self.__llm_model__)
 
     @property
-    def tokenizer(self) -> LLMTokenizer:
+    def tokenizer(self) -> _T:
         """The tokenizer to use for this LLM. This shouldn't be set at runtime, rather let OpenLLM handle it."""
-        # Run check for GPU
-        self.config.check_if_gpu_is_available(self.__llm_implementation__)
         if self.__llm_tokenizer__ is None:
             try:
                 self.__llm_tokenizer__ = self._bentomodel.custom_objects["tokenizer"]
@@ -675,18 +719,16 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
 
     def to_runner(
         self,
-        name: str | None = None,
         models: list[bentoml.Model] | None = None,
         max_batch_size: int | None = None,
         max_latency_ms: int | None = None,
         method_configs: dict[str, ModelSignatureDict | ModelSignature] | None = None,
         embedded: bool = False,
         scheduling_strategy: type[Strategy] | None = None,
-    ) -> bentoml.Runner:
+    ) -> LLMRunner:
         """Convert this LLM into a Runner.
 
         Args:
-            name: The name of the runner to generate. Optional as this will be generated based on the model_name.
             models: Any additional ``bentoml.Model`` to be included in this given models.
                     By default, this will be determined from the model_name.
             max_batch_size: The maximum batch size for the runner.
@@ -695,10 +737,13 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
             strategy: The strategy to use for this runner.
             embedded: Whether to run this runner in embedded mode.
             scheduling_strategy: Whether to create a custom scheduling strategy for this Runner.
+
+        NOTE: There are some difference between bentoml.models.get().to_runner() and LLM.to_runner(): 'name'.
+        - 'name': will be generated by OpenLLM, hence users don't shouldn't worry about this.
+            The generated name will be 'llm-<model-start-name>-runner' (ex: llm-dolly-v2-runner, llm-chatglm-runner)
         """
 
-        if name is None:
-            name = f"llm-{self.config.__openllm_start_name__}-runner"
+        name = f"llm-{self.config.__openllm_start_name__}-runner"
         models = models if models is not None else []
 
         # NOTE: The side effect of this is that will load the imported model during runner creation.
@@ -797,7 +842,7 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
                 (_Runnable,),
                 {
                     "SUPPORTED_RESOURCES": ("nvidia.com/gpu", "cpu")
-                    if self.__openllm_requires_gpu__
+                    if self.config.__openllm_requires_gpu__
                     else ("nvidia.com/gpu",),
                     "llm_type": self.llm_type,
                     "identifying_params": self.identifying_params,
@@ -828,7 +873,7 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
         return self.postprocess_generate(prompt, generated_result, **postprocess_kwargs)
 
 
-def Runner(start_name: str, **attrs: t.Any) -> bentoml.Runner:
+def Runner(start_name: str, **attrs: t.Any) -> LLMRunner:
     """Create a Runner for given LLM. For a list of currently supported LLM, check out 'openllm models'
 
     Args:
