@@ -31,28 +31,30 @@ import bentoml
 import openllm
 
 model = os.environ.get("OPENLLM_MODEL", "{__model_name__}")  # openllm: model name
+model_id = os.environ.get("OPENLLM_MODEL_ID", "{__model_id__}")  # openllm: model id
 
 llm_config = openllm.AutoConfig.for_model(model)
-runner = openllm.Runner(model, llm_config=llm_config)
+runner = openllm.Runner(model, model_id=model_id, llm_config=llm_config)
 
 svc = bentoml.Service(name=f"llm-{llm_config.__openllm_start_name__}-service", runners=[runner])
 
 
 @svc.api(
-    input=bentoml.io.JSON.from_sample(sample={"prompt": "", "llm_config": {}}),
-    output=bentoml.io.JSON.from_sample(sample={"responses": [], "configuration": {}}),
+    input=bentoml.io.JSON.from_sample(sample={"prompt": "", "llm_config": llm_config.model_dump()}),
+    output=bentoml.io.JSON.from_sample(sample={"responses": [], "configuration": llm_config.model_dump()}),
     route="/v1/generate",
 )
 async def generate_v1(input_dict: dict[str, t.Any]) -> openllm.GenerationOutput:
-    qa = openllm.GenerationInput.for_model(model)(**input_dict)
-    config = llm_config.model_construct_env(__llm_config__=qa.llm_config).model_dump()
-    responses = await runner.generate.async_run(qa.prompt, **config)
+    qa_inputs = openllm.GenerationInput.for_model(model)(**input_dict)
+    config = qa_inputs.llm_config.model_dump()
+    responses = await runner.generate.async_run(qa_inputs.prompt, **config)
     return openllm.GenerationOutput(responses=responses, configuration=config)
 
 
 @svc.api(input=bentoml.io.Text(), output=bentoml.io.JSON(), route="/v1/metadata")
 def metadata_v1(_: str) -> dict[str, t.Any]:
     return {
+        "model_id": model_id,
         "timeout": llm_config.__openllm_timeout__,
         "model_name": llm_config.__openllm_model_name__,
         "framework": llm_config.__openllm_env__.get_framework_env(),
