@@ -15,4 +15,7 @@
 from . import auto as auto
 from . import chatglm as chatglm
 from . import dolly_v2 as dolly_v2
+from . import falcon as falcon
 from . import flan_t5 as flan_t5
+from . import stablelm as stablelm
+from . import starcoder as starcoder
