@@ -26,9 +26,9 @@ import openllm
 from .configuration_auto import AutoConfig
 
 if t.TYPE_CHECKING:
-    import bentoml
+    from ..._llm import LLMRunner
 
-    ConfigModelOrderedDict = OrderedDict[type[openllm.LLMConfig], type[openllm.LLM]]
+    ConfigModelOrderedDict = OrderedDict[type[openllm.LLMConfig], type[openllm.LLM[t.Any, t.Any]]]
 else:
     ConfigModelOrderedDict = OrderedDict
 
@@ -47,11 +47,11 @@ class _BaseAutoLLMClass:
     def for_model(
         cls,
         model_name: str,
-        pretrained: str | None = None,
+        model_id: str | None = None,
         return_runner_kwargs: t.Literal[False] = ...,
         llm_config: openllm.LLMConfig | None = ...,
         **attrs: t.Any,
-    ) -> openllm.LLM:
+    ) -> openllm.LLM[t.Any, t.Any]:
         ...
 
     @t.overload
@@ -59,37 +59,42 @@ class _BaseAutoLLMClass:
     def for_model(
         cls,
         model_name: str,
-        pretrained: str | None = None,
+        model_id: str | None = None,
         return_runner_kwargs: t.Literal[True] = ...,
         llm_config: openllm.LLMConfig | None = ...,
         **attrs: t.Any,
-    ) -> tuple[openllm.LLM, dict[str, t.Any]]:
+    ) -> tuple[openllm.LLM[t.Any, t.Any], dict[str, t.Any]]:
         ...
 
     @classmethod
     def for_model(
         cls,
         model_name: str,
-        pretrained: str | None = None,
+        model_id: str | None = None,
         return_runner_kwargs: bool = False,
         llm_config: openllm.LLMConfig | None = ...,
         **attrs: t.Any,
-    ) -> openllm.LLM | tuple[openllm.LLM, dict[str, t.Any]]:
+    ) -> openllm.LLM[t.Any, t.Any] | tuple[openllm.LLM[t.Any, t.Any], dict[str, t.Any]]:
+        """The lower level API for creating a LLM instance.
+
+        ```python
+        >>> import openllm
+        >>> llm = openllm.AutoLLM.for_model("flan-t5")
+        ```
+        """
         runner_kwargs_name = [
-            "name",
             "models",
             "max_batch_size",
             "max_latency_ms",
             "method_configs",
             "embedded",
-            "scheduling_strategy",
         ]
         to_runner_attrs = {k: v for k, v in attrs.items() if k in runner_kwargs_name}
         if not isinstance(llm_config, openllm.LLMConfig):
             # The rest of kwargs is now passed to config
             llm_config = AutoConfig.for_model(model_name, **attrs)
         if type(llm_config) in cls._model_mapping.keys():
-            llm = cls._model_mapping[type(llm_config)].from_pretrained(pretrained, llm_config=llm_config, **attrs)
+            llm = cls._model_mapping[type(llm_config)].from_pretrained(model_id, llm_config=llm_config, **attrs)
             if not return_runner_kwargs:
                 return llm
             return llm, to_runner_attrs
@@ -99,23 +104,23 @@ class _BaseAutoLLMClass:
         )
 
     @classmethod
-    def create_runner(cls, model_name: str, pretrained: str | None = None, **attrs: t.Any) -> bentoml.Runner:
+    def create_runner(cls, model_name: str, model_id: str | None = None, **attrs: t.Any) -> LLMRunner:
         """
         Create a LLM Runner for the given model name.
 
         Args:
             model_name: The model name to instantiate.
-            pretrained: The pretrained model name to instantiate.
+            model_id: The pretrained model name to instantiate.
             **attrs: Additional keyword arguments passed along to the specific configuration class.
 
         Returns:
             A LLM instance.
         """
-        llm, runner_attrs = cls.for_model(model_name, pretrained, return_runner_kwargs=True, **attrs)
+        llm, runner_attrs = cls.for_model(model_name, model_id, return_runner_kwargs=True, **attrs)
         return llm.to_runner(**runner_attrs)
 
     @classmethod
-    def register(cls, config_class: type[openllm.LLMConfig], llm_class: type[openllm.LLM]):
+    def register(cls, config_class: type[openllm.LLMConfig], llm_class: type[openllm.LLM[t.Any, t.Any]]):
         """
         Register a new model for this class.
 
@@ -153,7 +158,10 @@ def getattribute_from_module(module: types.ModuleType, attr: t.Any) -> t.Any:
 
 
 class _LazyAutoMapping(ConfigModelOrderedDict):
-    """Based on transformers.models.auto.configuration_auto._LazyAutoMapping"""
+    """Based on transformers.models.auto.configuration_auto._LazyAutoMapping
+    This OrderedDict values() and keys() returns the list instead, so you don't
+    have to do list(mapping.values()) to get the list of values.
+    """
 
     def __init__(self, config_mapping: OrderedDict[str, str], model_mapping: OrderedDict[str, str]):
         self._config_mapping = config_mapping
@@ -166,7 +174,7 @@ class _LazyAutoMapping(ConfigModelOrderedDict):
         common_keys = set(self._config_mapping.keys()).intersection(self._model_mapping.keys())
         return len(common_keys) + len(self._extra_content)
 
-    def __getitem__(self, key: type[openllm.LLMConfig]) -> type[openllm.LLM]:
+    def __getitem__(self, key: type[openllm.LLMConfig]) -> type[openllm.LLM[t.Any, t.Any]]:
         if key in self._extra_content:
             return self._extra_content[key]
         model_type = self._reverse_config_mapping[key.__name__]
