@@ -13,7 +13,6 @@
 # limitations under the License.
 from __future__ import annotations
 
-import re
 import typing as t
 
 import bentoml
@@ -39,49 +38,27 @@ class InvalidScoreLogitsProcessor(LogitsProcessor):
         return scores
 
 
-def process_response(
-    response: str,
-    use_default_prompt_template: bool = True,
-):
-    response = response.strip()
-    if use_default_prompt_template:
-        response = response.replace("[[训练时间]]", "2023年")
-        punkts = [
-            [",", "，"],
-            ["!", "！"],
-            [":", "："],
-            [";", "；"],
-            ["\?", "？"],
-        ]
-        for item in punkts:
-            response = re.sub(r"([\u4e00-\u9fff])%s" % item[0], r"\1%s" % item[1], response)
-            response = re.sub(r"%s([\u4e00-\u9fff])" % item[0], r"%s\1" % item[1], response)
-    return response
-
-
-class ChatGLM(openllm.LLM):
+class ChatGLM(openllm.LLM["transformers.PreTrainedModel", "transformers.PreTrainedTokenizerFast"]):
     __openllm_internal__ = True
 
-    load_in_mha = False  # NOTE: disable bettertransformer for ChatGLM since it is already quantized
-
-    default_model = "THUDM/chatglm-6b-int4"
-
-    requirements = ["cpm_kernels", "sentencepiece"]
-
-    variants = ["THUDM/chatglm-6b", "THUDM/chatglm-6b-int8", "THUDM/chatglm-6b-int4"]
-
-    device = torch.device("cuda")
+    def llm_post_init(self):
+        self.device = torch.device("cuda")
 
     def import_model(
-        self, pretrained: str, tag: bentoml.Tag, *model_args: t.Any, tokenizer_kwds: dict[str, t.Any], **attrs: t.Any
+        self,
+        model_id: str,
+        tag: bentoml.Tag,
+        *model_args: t.Any,
+        tokenizer_kwds: dict[str, t.Any],
+        **attrs: t.Any,
     ) -> bentoml.Model:
         trust_remote_code = attrs.pop("trust_remote_code", True)
         return bentoml.transformers.save_model(
             tag,
-            transformers.AutoModel.from_pretrained(pretrained, trust_remote_code=trust_remote_code),
+            transformers.AutoModel.from_pretrained(model_id, trust_remote_code=trust_remote_code),
             custom_objects={
                 "tokenizer": transformers.AutoTokenizer.from_pretrained(
-                    pretrained, trust_remote_code=trust_remote_code, **tokenizer_kwds
+                    model_id, trust_remote_code=trust_remote_code, **tokenizer_kwds
                 )
             },
         )
@@ -114,7 +91,6 @@ class ChatGLM(openllm.LLM):
             "num_beams": num_beams,
             "top_p": top_p,
             "temperature": temperature,
-            "use_default_prompt_template": use_default_prompt_template,
             **attrs,
         }
 
@@ -126,7 +102,7 @@ class ChatGLM(openllm.LLM):
         if self.config.retain_history:
             assert chat_history is not None, "'retain_history' is True while there is no history provided."
             chat_history.append((prompt, generation_result))
-        return generation_result
+        return "".join(generation_result)
 
     @torch.inference_mode()
     def generate(self, prompt: str, use_default_prompt_template: bool = True, **attrs: t.Any) -> str:
@@ -149,4 +125,4 @@ class ChatGLM(openllm.LLM):
         )
         outputs = outputs.tolist()[0][len(inputs["input_ids"][0]) :]
         response = self.tokenizer.decode(outputs)
-        return process_response(response, use_default_prompt_template)
+        return self.model.process_response(response)
