@@ -14,6 +14,7 @@
 
 from __future__ import annotations
 
+import importlib
 import typing as t
 
 import bentoml
@@ -30,35 +31,52 @@ else:
     torch = openllm.utils.LazyLoader("torch", globals(), "torch")
 
 
-class Falcon(openllm.LLM):
+class Falcon(openllm.LLM["transformers.TextGenerationPipeline", "transformers.PreTrainedTokenizerFast"]):
     __openllm_internal__ = True
 
-    default_model = "tiiuae/falcon-7b"
-
-    requirements = ["einops"]
-
-    variants = ["tiiuae/falcon-7b", "tiiuae/falcon-40b", "tiiuae/falcon-7b-instruct", "tiiuae/falcon-40b-instruct"]
-
-    import_kwargs = {"torch_dtype": torch.bfloat16, "device_map": "auto"}
+    @property
+    def import_kwargs(self):
+        return {
+            "torch_dtype": torch.bfloat16,
+            "device_map": "auto" if torch.cuda.is_available() and torch.cuda.device_count() > 1 else None,
+        }
 
     def import_model(
-        self, pretrained: str, tag: bentoml.Tag, *model_args: t.Any, tokenizer_kwds: dict[str, t.Any], **attrs: t.Any
+        self, model_id: str, tag: bentoml.Tag, *model_args: t.Any, tokenizer_kwds: dict[str, t.Any], **attrs: t.Any
     ) -> bentoml.Model:
         trust_remote_code = attrs.pop("trust_remote_code", True)
         torch_dtype = attrs.pop("torch_dtype", torch.bfloat16)
         device_map = attrs.pop("device_map", "auto")
 
-        tokenizer = transformers.AutoTokenizer.from_pretrained(pretrained)
+        tokenizer = transformers.AutoTokenizer.from_pretrained(model_id)
         model = transformers.AutoModelForCausalLM.from_pretrained(
-            pretrained, trust_remote_code=trust_remote_code, torch_dtype=torch_dtype, device_map=device_map
-        )
-        config = transformers.AutoConfig.from_pretrained(pretrained, trust_remote_code=trust_remote_code)
-        transformers.AutoModelForCausalLM.register(config.__class__, model.__class__)
-        return bentoml.transformers.save_model(
-            tag,
-            transformers.pipeline("text-generation", model=model, tokenizer=tokenizer),
-            custom_objects={"tokenizer": tokenizer},
+            model_id,
+            trust_remote_code=trust_remote_code,
+            torch_dtype=torch_dtype,
+            device_map=device_map,
         )
+        try:
+            return bentoml.transformers.save_model(
+                tag,
+                model,
+                custom_objects={"tokenizer": tokenizer},
+                external_modules=[importlib.import_module(model.__module__)],
+            )
+        finally:
+            import gc
+
+            # NOTE: We need to free the cache after saving here so that we can load it back later on.
+            gc.collect()
+            torch.cuda.empty_cache()
+
+    def load_model(self, tag: bentoml.Tag, *args: t.Any, **attrs: t.Any) -> t.Any:
+        torch_dtype = attrs.pop("torch_dtype", torch.bfloat16)
+        device_map = attrs.pop("device_map", "auto")
+
+        _ref = bentoml.transformers.get(tag)
+
+        model = bentoml.transformers.load_model(_ref, device_map=device_map, torch_dtype=torch_dtype, **attrs)
+        return transformers.pipeline("text-generation", model=model, tokenizer=_ref.custom_objects["tokenizer"])
 
     def sanitize_parameters(
         self,
@@ -67,21 +85,24 @@ class Falcon(openllm.LLM):
         top_k: int | None = None,
         num_return_sequences: int | None = None,
         eos_token_id: int | None = None,
-        use_default_prompt_template: bool = True,
+        use_default_prompt_template: bool = False,
         **attrs: t.Any,
     ) -> tuple[str, dict[str, t.Any], dict[str, t.Any]]:
         if use_default_prompt_template:
-            prompt_variables = {
-                k: v
-                for k, v in attrs.items()
-                if k in default_formatter.extract_template_variables(DEFAULT_PROMPT_TEMPLATE)
-            }
+            template_variables = default_formatter.extract_template_variables(DEFAULT_PROMPT_TEMPLATE)
+            prompt_variables = {k: v for k, v in attrs.items() if k in template_variables}
             if "instruction" in prompt_variables:
                 raise RuntimeError(
                     "'instruction' should be passed as the first argument instead of "
                     "kwargs when 'use_default_prompt_template=True'"
                 )
-            prompt_text = DEFAULT_PROMPT_TEMPLATE.format(instruction=prompt, **prompt_variables)
+            try:
+                prompt_text = DEFAULT_PROMPT_TEMPLATE.format(instruction=prompt, **prompt_variables)
+            except KeyError as e:
+                raise RuntimeError(
+                    f"Missing variable '{e.args[0]}' (required: {template_variables}) in the prompt template. "
+                    "Use 'use_default_prompt_template=False' to disable the default prompt template."
+                )
         else:
             prompt_text = prompt
 
@@ -98,11 +119,7 @@ class Falcon(openllm.LLM):
     def postprocess_generate(self, prompt: str, generation_result: t.Sequence[dict[str, t.Any]], **_: t.Any) -> str:
         return "\n".join([i["generated_text"] for i in generation_result])
 
-    @torch.inference_mode()
     def generate(self, prompt: str, **attrs: t.Any) -> list[dict[str, t.Any]]:
-        # NOTE: MK tokenizer into this pipeline, since we can't inject tokenizer at load time
-        self.model.tokenizer = self.tokenizer
-
         eos_token_id = attrs.pop("eos_token_id", self.tokenizer.eos_token_id)
 
         # NOTE: our model here is the pipeline
