@@ -16,7 +16,7 @@ from __future__ import annotations
 import openllm
 
 START_FLAN_T5_COMMAND_DOCSTRING = """\
-Run a LLMServer for FLAN-T5 model and variants.
+Run a LLMServer for FLAN-T5 model.
 
 \b
 > See more information about FLAN-T5 at [huggingface/transformers](https://huggingface.co/docs/transformers/model_doc/flan-t5)
@@ -34,7 +34,11 @@ By default, this model will use the PyTorch model for inference. However, this m
 
 \b
 FLAN-T5 Runner will use google/flan-t5-large as the default model. To change any to any other FLAN-T5
-saved pretrained, or a fine-tune FLAN-T5, provide ``OPENLLM_FLAN_T5_PRETRAINED='google/flan-t5-xxl'``
+saved pretrained, or a fine-tune FLAN-T5, provide ``OPENLLM_FLAN_T5_MODEL_ID='google/flan-t5-xxl'``
+or provide `--model-id` flag when running ``openllm start flan-t5``:
+
+\b
+$ openllm start flan-t5 --model-id google/flan-t5-xxl
 """
 
 DEFAULT_PROMPT_TEMPLATE = """Answer the following question:\nQuestion: {instruction}\nAnswer:"""
@@ -47,6 +51,18 @@ class FlanT5Config(openllm.LLMConfig):
     Refer to [FLAN-T5's page](https://huggingface.co/docs/transformers/model_doc/flan-t5) for more information.
     """
 
+    __config__ = {
+        "url": "https://huggingface.co/docs/transformers/model_doc/flan-t5",
+        "default_id": "google/flan-t5-large",
+        "model_ids": [
+            "google/flan-t5-small",
+            "google/flan-t5-base",
+            "google/flan-t5-large",
+            "google/flan-t5-xl",
+            "google/flan-t5-xxl",
+        ],
+    }
+
     class GenerationConfig:
         temperature: float = 0.9
         max_new_tokens: int = 2048
