@@ -20,20 +20,13 @@ import openllm
 from ..._prompt import default_formatter
 from .configuration_flan_t5 import DEFAULT_PROMPT_TEMPLATE
 
+if t.TYPE_CHECKING:
+    import transformers  # noqa
 
-class TFFlanT5(openllm.LLM):
+
+class TFFlanT5(openllm.LLM["transformers.TFT5ForConditionalGeneration", "transformers.T5TokenizerFast"]):
     __openllm_internal__ = True
 
-    default_model: str = "google/flan-t5-large"
-
-    variants = [
-        "google/flan-t5-small",
-        "google/flan-t5-base",
-        "google/flan-t5-large",
-        "google/flan-t5-xl",
-        "google/flan-t5-xxl",
-    ]
-
     def sanitize_parameters(
         self,
         prompt: str,
