@@ -21,7 +21,10 @@ from transformers import StoppingCriteria, StoppingCriteriaList
 import openllm
 
 from ..._prompt import default_formatter
-from .configuration_stablelm import DEFAULT_PROMPT_TEMPLATE
+from .configuration_stablelm import DEFAULT_PROMPT_TEMPLATE, SYSTEM_PROMPT
+
+if t.TYPE_CHECKING:
+    import transformers  # noqa
 
 
 class StopOnTokens(StoppingCriteria):
@@ -39,25 +42,20 @@ else:
 logger = logging.getLogger(__name__)
 
 
-class StableLM(openllm.LLM):
+class StableLM(openllm.LLM["transformers.GPTNeoXForCausalLM", "transformers.GPTNeoXTokenizerFast"]):
     __openllm_internal__ = True
 
-    load_in_mha = False
+    def llm_post_init(self):
+        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
+        self.load_in_mha = True if not torch.cuda.is_available() else False
 
-    default_model = "StabilityAI/stablelm-tuned-alpha-3b"
-
-    variants = [
-        "StabilityAI/stablelm-tuned-alpha-3b",
-        "StabilityAI/stablelm-tuned-alpha-7b",
-        "StabilityAI/stablelm-base-alpha-3b",
-        "StabilityAI/stablelm-base-alpha-7b",
-    ]
-
-    import_kwargs = {
-        "torch_dtype": torch.float16,
-        "load_in_8bit": True if torch.cuda.is_available() and torch.cuda.device_count() == 1 else False,
-        "device_map": "auto",
-    }
+    @property
+    def import_kwargs(self):
+        return {
+            "torch_dtype": torch.float16 if torch.cuda.is_available() else torch.float32,
+            "load_in_8bit": False,
+            "device_map": "auto" if torch.cuda.is_available() and torch.cuda.device_count() > 1 else None,
+        }
 
     def sanitize_parameters(
         self,
@@ -69,7 +67,7 @@ class StableLM(openllm.LLM):
         use_default_prompt_template: bool = True,
         **attrs: t.Any,
     ) -> tuple[str, dict[str, t.Any], dict[str, t.Any]]:
-        if "tuned" in self._pretrained and use_default_prompt_template:
+        if "tuned" in self._model_id and use_default_prompt_template:
             prompt_variables = {
                 k: v
                 for k, v in attrs.items()
@@ -80,7 +78,8 @@ class StableLM(openllm.LLM):
                     "'instruction' should be passed as the first argument "
                     "instead of kwargs when 'use_default_prompt_template=True'"
                 )
-            prompt_text = DEFAULT_PROMPT_TEMPLATE.format(instruction=prompt, **prompt_variables)
+            system_prompt = prompt_variables.pop("system_prompt", SYSTEM_PROMPT)
+            prompt_text = DEFAULT_PROMPT_TEMPLATE.format(instruction=prompt, system_prompt=system_prompt)
         else:
             prompt_text = prompt
 
@@ -96,13 +95,7 @@ class StableLM(openllm.LLM):
     def postprocess_generate(self, prompt: str, generation_result: list[str], **_: t.Any) -> str:
         return generation_result[0]
 
-    @torch.inference_mode()
     def generate(self, prompt: str, **attrs: t.Any) -> list[str]:
-        if torch.cuda.is_available():
-            self.model.cuda()
-        if not self.model.is_loaded_in_8bit:
-            self.model.half()
-
         generation_kwargs = {
             "do_sample": True,
             "generation_config": self.config.model_construct_env(**attrs).to_generation_config(),
@@ -110,6 +103,9 @@ class StableLM(openllm.LLM):
             "stopping_criteria": StoppingCriteriaList([StopOnTokens()]),
         }
 
-        inputs = t.cast("torch.Tensor", self.tokenizer(prompt, return_tensors="pt")).to(self.model.device)
+        if torch.cuda.is_available():
+            self.model.cuda()
+
+        inputs = t.cast("torch.Tensor", self.tokenizer(prompt, return_tensors="pt")).to(self.device)
         tokens = self.model.generate(**inputs, **generation_kwargs)
         return [self.tokenizer.decode(tokens[0], skip_special_tokens=True)]
