@@ -20,8 +20,6 @@ import bentoml
 
 import openllm
 
-from .configuration_starcoder import DEFAULT_PROMPT_TEMPLATE
-
 if t.TYPE_CHECKING:
     import torch
     import transformers
@@ -39,40 +37,34 @@ EOD = "<|endoftext|>"
 FIM_INDICATOR = "<FILL_HERE>"
 
 
-class StarCoder(openllm.LLM):
+class StarCoder(openllm.LLM["transformers.GPTBigCodeForCausalLM", "transformers.GPT2TokenizerFast"]):
     __openllm_internal__ = True
 
-    default_model = "bigcode/starcoder"
+    def llm_post_init(self):
+        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
 
-    requirements = ["bitandbytes"]
-
-    variants = ["bigcode/starcoder", "bigcode/starcoderbase"]
-
-    device = torch.device("cuda")
-
-    import_kwargs = {
-        "_tokenizer_padding_side": "left",
-        "device_map": "auto",
-        "load_in_8bit": True,
-        "torch_dtype": torch.float16,
-    }
+    @property
+    def import_kwargs(self):
+        return {
+            "_tokenizer_padding_side": "left",
+            "device_map": "auto" if torch.cuda.is_available() and torch.cuda.device_count() > 1 else None,
+            "load_in_8bit": True if torch.cuda.device_count() > 1 else False,
+            "torch_dtype": torch.float16,
+        }
 
     def import_model(
         self,
-        pretrained: str,
+        model_id: str,
         tag: bentoml.Tag,
         *model_args: t.Any,
         tokenizer_kwds: dict[str, t.Any],
         **attrs: t.Any,
     ) -> bentoml.Model:
-        trust_remote_code = attrs.pop("trust_remote_code", True)
         torch_dtype = attrs.pop("torch_dtype", torch.float16)
         load_in_8bit = attrs.pop("load_in_8bit", True)
         device_map = attrs.pop("device_map", "auto")
 
-        tokenizer = transformers.AutoTokenizer.from_pretrained(
-            pretrained, trust_remote_code=trust_remote_code, **tokenizer_kwds
-        )
+        tokenizer = transformers.AutoTokenizer.from_pretrained(model_id, **tokenizer_kwds)
         tokenizer.add_special_tokens(
             {
                 "additional_special_tokens": [EOD, FIM_PREFIX, FIM_MIDDLE, FIM_SUFFIX, FIM_PAD],
@@ -81,14 +73,8 @@ class StarCoder(openllm.LLM):
         )
 
         model = transformers.AutoModelForCausalLM.from_pretrained(
-            pretrained,
-            torch_dtype=torch_dtype,
-            load_in_8bit=load_in_8bit,
-            device_map=device_map,
-            trust_remote_code=trust_remote_code,
-            **attrs,
+            model_id, torch_dtype=torch_dtype, load_in_8bit=load_in_8bit, device_map=device_map, **attrs
         )
-
         try:
             return bentoml.transformers.save_model(tag, model, custom_objects={"tokenizer": tokenizer})
         finally:
@@ -105,21 +91,17 @@ class StarCoder(openllm.LLM):
         top_p: float | None = None,
         max_new_tokens: int | None = None,
         repetition_penalty: float | None = None,
-        use_default_prompt_template: bool = True,
         **attrs: t.Any,
     ) -> tuple[str, dict[str, t.Any], dict[str, t.Any]]:
-        if use_default_prompt_template:
-            fim_mode = FIM_INDICATOR in prompt
-            prefix, suffix = None, None
-            if fim_mode:
-                try:
-                    prefix, suffix = prompt.split(FIM_INDICATOR)
-                except Exception as err:
-                    logger.error("Error while processing prompt with FIM mode:\n", exc_info=err)
-                    raise ValueError(f"Only one {FIM_INDICATOR} allowed in prompt") from err
-                prompt_text = f"{FIM_PREFIX}{prefix}{FIM_SUFFIX}{suffix}{FIM_MIDDLE}"
-            else:
-                prompt_text = prompt
+        fim_mode = FIM_INDICATOR in prompt
+        prefix, suffix = None, None
+        if fim_mode:
+            try:
+                prefix, suffix = prompt.split(FIM_INDICATOR)
+            except Exception as err:
+                logger.error("Error while processing prompt with FIM mode:\n", exc_info=err)
+                raise ValueError(f"Only one {FIM_INDICATOR} allowed in prompt") from err
+            prompt_text = f"{FIM_PREFIX}{prefix}{FIM_SUFFIX}{suffix}{FIM_MIDDLE}"
         else:
             prompt_text = prompt
 
@@ -134,9 +116,6 @@ class StarCoder(openllm.LLM):
             **attrs,
         }
 
-        if use_default_prompt_template:
-            prompt_text = DEFAULT_PROMPT_TEMPLATE.format(instruction=prompt_text)
-
         return prompt_text, generation_config, {}
 
     def postprocess_generate(self, prompt: str, generation_result: t.Sequence[str], **_: t.Any) -> str:
@@ -148,8 +127,12 @@ class StarCoder(openllm.LLM):
         result_tensor = self.model.generate(
             inputs,
             do_sample=True,
+            pad_token_id=self.tokenizer.eos_token_id,
+            # eos_token_id=self.tokenizer.convert_tokens_to_ids("<|end|>"), # NOTE: this is for finetuning starcoder
             generation_config=self.config.model_construct_env(**attrs).to_generation_config(),
         )
         # TODO: We will probably want to return the tokenizer here so that we can manually process this
         # return (skip_special_tokens=False, clean_up_tokenization_spaces=False))
-        return [self.tokenizer.decode(result_tensor[0])]
+        return self.tokenizer.batch_decode(
+            result_tensor[0], skip_special_tokens=True, clean_up_tokenization_spaces=True
+        )
