@@ -18,6 +18,8 @@ import asyncio
 import logging
 import typing as t
 
+import openllm
+
 from .base import BaseAsyncClient, BaseClient
 
 if t.TYPE_CHECKING:
@@ -54,6 +56,18 @@ class GrpcClientMixin:
         except KeyError:
             raise RuntimeError("Malformed service endpoint. (Possible malicious)")
 
+    @property
+    def model_id(self) -> str:
+        try:
+            return self._metadata.json.struct_value.fields["model_id"].string_value
+        except KeyError:
+            raise RuntimeError("Malformed service endpoint. (Possible malicious)")
+
+    def postprocess(self, result: Response) -> openllm.GenerationOutput:
+        from google.protobuf.json_format import MessageToDict
+
+        return openllm.GenerationOutput(**MessageToDict(result.json, preserving_proto_field_name=True))
+
 
 class GrpcClient(GrpcClientMixin, BaseClient, client_type="grpc"):
     def __init__(self, address: str, timeout: int = 30):
