@@ -18,6 +18,8 @@ import logging
 import typing as t
 from urllib.parse import urlparse
 
+import openllm
+
 from .base import BaseAsyncClient, BaseClient
 
 logger = logging.getLogger(__name__)
@@ -33,6 +35,13 @@ class HTTPClientMixin:
         except KeyError:
             raise RuntimeError("Malformed service endpoint. (Possible malicious)")
 
+    @property
+    def model_id(self) -> str:
+        try:
+            return self._metadata["model_name"]
+        except KeyError:
+            raise RuntimeError("Malformed service endpoint. (Possible malicious)")
+
     @property
     def framework(self) -> t.Literal["pt", "flax", "tf"]:
         try:
@@ -47,6 +56,9 @@ class HTTPClientMixin:
         except KeyError:
             raise RuntimeError("Malformed service endpoint. (Possible malicious)")
 
+    def postprocess(self, result: dict[str, t.Any]) -> openllm.GenerationOutput:
+        return openllm.GenerationOutput(**result)
+
 
 class HTTPClient(HTTPClientMixin, BaseClient):
     def __init__(self, address: str, timeout: int = 30):
